/*
 * Copyright (c) 2000-2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * PROPRIETARY/CONFIDENTIAL
 *
 * This software is the confidential and proprietary information of
 * SAMSUNG ELECTRONICS ("Confidential Information").
 * You shall not disclose such Confidential Information and shall
 * use it only in accordance with the terms of the license agreement
 * you entered into with SAMSUNG ELECTRONICS.
 * SAMSUNG make no representations or warranties about the suitability
 * of the software, either express or implied, including but not
 * limited to the implied warranties of merchantability, fitness for
 * a particular purpose, or non-infringement.
 * SAMSUNG shall not be liable for any damages suffered by licensee as
 * a result of using, modifying or distributing this software or its derivatives.
 */

#ifndef IAP_GALAXYAPPS_PRODUCT_H
#define IAP_GALAXYAPPS_PRODUCT_H

#include <tizen.h>
#include <tizen_error_extension.h>


#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file iap_galaxyapps.h
 */

/**
 * @brief In-App Purchase handle.
 * @since_tizen 2.3.2
 */
typedef struct iap_galaxyapps_s* iap_galaxyapps_h;

/**
 * @brief Enumeration for In-App Purchase Error.
 * @since_tizen 2.3.2
 */
typedef enum
{
	IAP_GALAXYAPPS_ERROR_NONE = TIZEN_ERROR_NONE, /**< Successful */
	IAP_GALAXYAPPS_ERROR_PAYMENT_IS_CANCELED = TIZEN_ERROR_CANCELED, /**< Payment canceled */
	IAP_GALAXYAPPS_ERROR_NETWORK_NOT_AVAILABLE = TIZEN_ERROR_NETWORK_DOWN, /** Error network is not available. */
	IAP_GALAXYAPPS_ERROR_IO_ERROR = TIZEN_ERROR_IO_ERROR, /**< IOException */
	IAP_GALAXYAPPS_ERROR_TIMED_OUT = TIZEN_ERROR_TIMED_OUT, /**< Timeout exception */
	IAP_GALAXYAPPS_ERROR_INITIALIZATION = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x01, /**< Failure during IAP initialization */
	IAP_GALAXYAPPS_ERROR_NEED_APP_UPGRADE = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x02, /**< Samsung IAP upgrade is required. */
	IAP_GALAXYAPPS_ERROR_COMMON = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x03, /**<  Error while running IAP */
	IAP_GALAXYAPPS_ERROR_ALREADY_PURCHASED = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x04, /**<  Error when a non-consumable product is repurchased or a subscription product is repurchased before the product expiration date. */
	IAP_GALAXYAPPS_ERROR_REQUEST_PAYMENT_WITHOUT_INFO = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x05, /**< Error when payment is requested without bundle information. */
	IAP_GALAXYAPPS_ERROR_PRODUCT_DOES_NOT_EXIST = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x06, /**< Error when the requested item list is not available. */
	IAP_GALAXYAPPS_ERROR_CONFIRM_INBOX = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x07, /** The payment result is not received after requesting payment from the server, and the purchased item list is not confirmed */
	IAP_GALAXYAPPS_ERROR_NOT_EXIST_LOCAL_PRICE = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x08, /** The item is not for sale in the country */
	IAP_GALAXYAPPS_ERROR_NOT_AVAILABLE_SHOP = TIZEN_ERROR_IN_APP_PURCHASE_GALAXYAPPS | 0x0a, /** IAP is not serviced in the country */

	IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER = TIZEN_ERROR_INVALID_PARAMETER, /**< Invalid parameter */
	IAP_GALAXYAPPS_ERROR_KEY_NOT_FOUND = TIZEN_ERROR_KEY_NOT_AVAILABLE, /**< Specified key is not found */
	IAP_GALAXYAPPS_ERROR_NOT_SUPPORTED_DEVICE = TIZEN_ERROR_NOT_SUPPORTED, /**< The device does not support IAP */
	IAP_GALAXYAPPS_ERROR_OUT_OF_MEMORY = TIZEN_ERROR_OUT_OF_MEMORY, /**< Out of memory */
	IAP_GALAXYAPPS_ERROR_PERMISSION_DENIED = TIZEN_ERROR_PERMISSION_DENIED,	/**< Permission denied */
} iap_galaxyapps_error_e;

/**
 * @brief Enumeration for mode
 * @since_tizen 2.3.2
 *
 * @remarks Be sure to set #IAP_COMMERCIAL_MODE when uploading an application for distribution so that actual payments from customers can be completed
 *
 * @see iap_galaxyapps_start_payment()
 * @see iap_galaxyapps_get_item_list()
 */
typedef enum
{
	IAP_GALAXYAPPS_COMMERCIAL_MODE = 0,		/**< Commercial mode */
	IAP_GALAXYAPPS_SUCCESS_TEST_MODE = 1,	/**< Test mode and always success */
	IAP_GALAXYAPPS_FAILURE_TEST_MODE = -1,	/**< Test mode and always failure */
} iap_galaxyapps_mode_e;

/**
 * @brief Called when the reply of the In-App Purchase request is delivered.
 * @since_tizen 2.3.2
 *
 * @remarks The @a reply must not be deallocated by the application.
 *
 * @param[in] reply The In-App Purchase handle in which the results of the callee are contained
 * @param[in] result The result code of the In-App Purchase request such as
 *				#IAP_GALAXYAPPS_ERROR_NONE Successful
 *				#IAP_GALAXYAPPS_ERROR_PAYMENT_IS_CANCELED Payment canceled
 *				#IAP_GALAXYAPPS_ERROR_INITIALIZATION Failure during IAP initialization
 *				#IAP_GALAXYAPPS_ERROR_NEED_APP_UPGRADE Samsung IAP upgrade is required
 *				#IAP_GALAXYAPPS_ERROR_COMMON Error while running IAP
 *				#IAP_GALAXYAPPS_ERROR_ALREADY_PURCHASED Error when a non-consumable product is repurchased or a subscription product is repurchased before the product expiration date.
 *				#IAP_GALAXYAPPS_ERROR_REQUEST_PAYMENT_WITHOUT_INFO Error when payment is requested without bundle information.
 *				#IAP_GALAXYAPPS_ERROR_PRODUCT_DOES_NOT_EXIST Error when the requested item list is not available.
 *				#IAP_GALAXYAPPS_ERROR_CONFIRM_INBOX The payment result is not received after requesting payment from the server, and the purchased item list is not confirmed
 *				#IAP_GALAXYAPPS_ERROR_NETWORK_NOT_AVAILABLE Error network is not available.
 *				#IAP_GALAXYAPPS_ERROR_IO_ERROR IOException
 *				#IAP_GALAXYAPPS_ERROR_TIMED_OUT Timeout exception
 *				#IAP_GALAXYAPPS_ERROR_NOT_EXIST_LOCAL_PRICE The item is not for sale in the country
 *				#IAP_GALAXYAPPS_ERROR_NOT_AVAILABLE_SHOP IAP is not serviced in the country
 * @param[in] user_data	The user data passed from the callback registration function
 * @pre When the callee replies to the In-App Purchase request, this callback will be invoked.
 * @see iap_galaxyapps_start_payment()
 * @see iap_galaxyapps_get_item_list()
 * @see iap_galaxyapps_get_purchased_item_list()
 */
typedef void (*iap_galaxyapps_reply_cb) (iap_galaxyapps_h reply, iap_galaxyapps_error_e result, void *user_data);

/**
 * @brief Called to retrieve all item information contained in the handle.
 * @since_tizen 2.3.2
 *
 * @param[in] handle The In-App Purchase handle
 * @param[in] user_data The user data passed from the foreach function
 * @return @c true to continue with the next iteration of the loop, otherwise @c false to break out of the loop
 * @pre	iap_galaxyapps_foreach_item_info() will invoke this callback.
 * @see	iap_galaxyapps_foreach_item_info()
 */
typedef bool (*iap_galaxyapps_item_info_cb)(iap_galaxyapps_h handle, void *user_data);

/**
 * @brief Purchases an item on Samsung Galaxy Apps
 * @since_tizen 2.3.2
 * @privlevel	public
 * @privilege	%http://tizen.org/privilege/billing
 *
 * @param[in] item_id  the item id to purchase
 * @param[in] mode	It can be possible to experience the entire process 'risk free' by enabling the IAP Test Mode when developing apps.
 *					Be sure to set #IAP_COMMERCIAL_MODE when uploading an application for distribution so that actual payments from customers can be completed.
 * @param[in] reply_cb The result callback function
 * @param[in] user_data The user data passed to the callback function
 * @return @c 0 on success, otherwise a negative error value
 * @retval #IAP_GALAXYAPPS_ERROR_NONE Successful
 * @retval #IAP_GALAXYAPPS_ERROR_INITIALIZATION Failure during IAP initialization
 * @retval #IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #IAP_GALAXYAPPS_ERROR_NOT_SUPPORTED_DEVICE The device does not support IAP
 * @retval #IAP_GALAXYAPPS_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #IAP_GALAXYAPPS_ERROR_PERMISSION_DENIED Permission denied
 * @see	iap_galaxyapps_mode_e
 * @see	iap_galaxyapps_reply_cb()
 */
int iap_galaxyapps_start_payment(const char *item_id, iap_galaxyapps_mode_e mode, iap_galaxyapps_reply_cb reply_cb, void *user_data);

/**
 * @brief Gets the list of a specified subset of the items on Samsung Galaxy Apps offered for sale by caller app
 * @since_tizen 2.3.2
 * @privlevel	public
 * @privilege	%http://tizen.org/privilege/billing
 *
 * @param[in] start_number the index of first item on the list
 * @param[in] end_number the index of last item on the list
 * @param[in] item_type the item type such as 00, 01, 02 and 10\n
 *				00: Non-consumable, 01: Consumable, 02: Subscription, 10: All
 * @param[in] mode	It can be possible to experience the entire process 'risk free' by enabling the IAP Test Mode when developing apps.
 *					Be sure to set #IAP_COMMERCIAL_MODE when uploading an application for distribution so that actual payments from customers can be completed.
 * @param[in] reply_cb The result callback function
 * @param[in] user_data The user data passed to the callback function
 * @return @c 0 on success, otherwise a negative error value
 * @retval #IAP_GALAXYAPPS_ERROR_NONE Successful
 * @retval #IAP_GALAXYAPPS_ERROR_INITIALIZATION Failure during IAP initialization
 * @retval #IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #IAP_GALAXYAPPS_ERROR_NOT_SUPPORTED_DEVICE The device does not support IAP
 * @retval #IAP_GALAXYAPPS_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #IAP_GALAXYAPPS_ERROR_PERMISSION_DENIED Permission denied
 * @see	iap_galaxyapps_mode_e
 * @see	iap_galaxyapps_reply_cb()
 */
int iap_galaxyapps_get_item_list(int start_number, int end_number, const char *item_type, iap_galaxyapps_mode_e mode, iap_galaxyapps_reply_cb reply_cb, void *user_data);

/**
 * @brief Gets the list of the purchased items on Samsung Galaxy Apps
 * @since_tizen 2.3.2
 * @privlevel	public
 * @privilege	%http://tizen.org/privilege/billing
 *
 * @param[in] start_number the index of first item on the list. it starts from 1.
 * @param[in] end_number the index of last item on the list
 * @param[in] start_date the start date of the requested inbox list. the date format is "yyyymmdd". ex) "20160301"
 * @param[in] end_date the end date of the requested inbox list. the date format is "yyyymmdd". ex) "20160331"
 * @param[in] reply_cb The result callback function
 * @param[in] user_data The user data passed to the callback function
 * @return @c 0 on success, otherwise a negative error value
 * @retval #IAP_GALAXYAPPS_ERROR_NONE Successful
 * @retval #IAP_GALAXYAPPS_ERROR_INITIALIZATION Failure during IAP initialization
 * @retval #IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #IAP_GALAXYAPPS_ERROR_NOT_SUPPORTED_DEVICE The device does not support IAP
 * @retval #IAP_GALAXYAPPS_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #IAP_GALAXYAPPS_ERROR_PERMISSION_DENIED Permission denied
 * @see	iap_galaxyapps_reply_cb()
 */
int iap_galaxyapps_get_purchased_item_list(int start_number, int end_number, const char *start_date, const char *end_date, iap_galaxyapps_reply_cb reply_cb, void *user_data);

/**
 * @brief Gets the list of the purchased items among @c item_ids on Samsung Galaxy Apps
 * @since_tizen 2.3.2
 * @privlevel	public
 * @privilege	%http://tizen.org/privilege/billing
 *
 * @param[in] item_ids	Item IDs seperated by comma(,)
 * @param[in] reply_cb The result callback function
 * @param[in] user_data The user data passed to the callback function
 * @return @c 0 on success, otherwise a negative error value
 * @retval #IAP_GALAXYAPPS_ERROR_NONE Successful
 * @retval #IAP_GALAXYAPPS_ERROR_INITIALIZATION Failure during IAP initialization
 * @retval #IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #IAP_GALAXYAPPS_ERROR_NOT_SUPPORTED_DEVICE The device does not support IAP
 * @retval #IAP_GALAXYAPPS_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #IAP_GALAXYAPPS_ERROR_PERMISSION_DENIED Permission denied
 * @see	iap_galaxyapps_reply_cb()
 */
int iap_galaxyapps_get_purchased_item_list_by_item_ids(const char *item_ids, iap_galaxyapps_reply_cb reply_cb, void *user_data);

/**
 * @brief Gets value associated with the given @a key
 * @since_tizen 2.3.2
 *
 * @param[in] handle The In-App Purchase handle
 * @param[in] key The name of the extra data
 * @param[out] value The value associated with the given key
 * @return @c 0 on success, otherwise a negative error value
 * @retval #IAP_GALAXYAPPS_ERROR_NONE Successful
 * @retval #IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #IAP_GALAXYAPPS_ERROR_KEY_NOT_FOUND Specified key not found
 * @see iap_galaxyapps_foreach_item_info()
 */
int iap_galaxyapps_get_value(iap_galaxyapps_h handle, const char *key, char **value);

/**
 * @brief Retrieves all items contained in handle.
 * @details This function calls iap_item_info_cb() repeatedly for each item information contained in @a handle. \n
 *		If the iap_item_info_cb() callback function returns @c false, then iteration will be finished.
 * @since_tizen 2.3.2
 *
 * @param[in] handle The In-App Purchase handle
 * @param[in] callback The iteration callback function
 * @param[in] user_data The user data to be passed to the callback function
 * @return @c 0 on success, otherwise a negative error value
 * @retval #IAP_GALAXYAPPS_ERROR_NONE Successful
 * @retval #IAP_GALAXYAPPS_ERROR_INVALID_PARAMETER Invalid parameter
 * @post This function invokes iap_item_info_cb().
 * @see iap_galaxyapps_item_info_cb()
 */
int iap_galaxyapps_foreach_item_info(iap_galaxyapps_h handle, iap_galaxyapps_item_info_cb callback, void *user_data);

#ifdef __cplusplus
}
#endif

#endif	// IAP_GALAXYAPPS_PRODUCT_H
