#!/bin/sh

########## START DEFINE FUNCTIONS ##########

#
# Reloading All Smack Rules (.rule & Rule DB)
#
all_reload(){
	echo "Reloading All Smack rules..."
	/bin/rm -f /etc/smack/accesses.d/*.efl
	cd /etc/smack/accesses2.d/
	/usr/bin/find *.rule -exec /usr/bin/smack-policy-gen {} \;
	cd -

	/bin/sync
	/etc/init.d/smack-utils reload

	db_reload
}

#
# Install Application Rule File (.rule)
# Param : Package ID
#
install_pkg(){
	echo "Installing $1 rule file"
	/bin/rm -f /etc/smack/accesses.d/$1.efl
	cd /etc/smack/accesses2.d/
	/usr/bin/smack-policy-gen $1.rule
	cd -
	/bin/sync
	/usr/bin/smackload /etc/smack/accesses.d/$1.efl
}

#
# Uninstall Application Rule File (.rule)
# Param : Package ID
#
uninstall_pkg(){
	echo "Uninstalling $1 rule file"
	/usr/bin/smackload -c /etc/smack/accesses.d/$1.efl
	/bin/rm -f /etc/smack/accesses.d/$1.efl
	/bin/sync
}

#
# Updating Application Rule File (.rule)
# Param : Package ID
#
update_pkg(){
	echo "Updating $1 rule file"
	/usr/bin/smackload -c /etc/smack/accesses.d/$1.efl
	/bin/rm -f /etc/smack/accesses.d/$1.efl
	cd /etc/smack/accesses2.d/
	/usr/bin/smack-policy-gen $1.rule
	cd -
	/bin/sync
	/usr/bin/smackload /etc/smack/accesses.d/$1.efl
}

#
# Reloading Smack Rule DB
#
db_reload(){
	echo "Reloading Smack Rule DB..."
	/bin/bash -c '/usr/bin/sqlite3 /opt/dbspace/.rules-db.db3 < /usr/share/privilege-control/db/load-rules-db.sql | /usr/bin/smack_rules_buffer > /smack/change-rule'
}

#
# Printing Help Message (Usage)
#
help(){
	echo "usage: smack_reload.sh [command] [args..]"
	echo "commands are:"
	echo "  all(or empty) : Reloading all Smack rules (no args)"
	echo "  install : Install Application Smack rules (args=pkgIDs)"
	echo "  uninstall : Uninstall Application Smack rules (args=pkgIDs)"
	echo "  update : Updating Application Smack rules (args=pkgIDs)"
	echo "  db : Reloading Smack Rule DB only (no args)"
}
########## END DEFINE FUNCTIONS ##########

op=$1 # Get a operation value from the 1st parameter

if [ "$op" = "" ]; then
	all_reload
elif [ "$op" != "" ]
then
	if [ "$op" = "--help" ]; then
		help
	elif [ "$op" = "all" ]; then
		all_reload
	elif [ "$op" = "install" -o "$op" = "uninstall" -o "$op" = "update" ]; then
		if [ "$2" = "" ]; then
			help
		elif [ "$op" = "install" ]; then
			while [ "$2" != "" ]
			do
				install_pkg $2
				shift 1
			done
			db_reload
		elif [ "$op" = "uninstall" ]; then
			while [ "$2" != "" ]
			do
				uninstall_pkg $2
				shift 1
			done
			db_reload
		elif [ "$op" = "update" ]; then
			while [ "$2" != "" ]
			do
				update_pkg $2
				shift 1
			done
			db_reload
		fi
	elif [ "$op" = "db" ]; then
		db_reload
	else
		help
	fi
fi

